# ----------------
# 한글을 자음/모음 분리
# ----------------
INITIALS = list(map(lambda x: chr(x), [
    0x3131, 0x3132, 0x3134, 0x3137, 0x3138, 0x3139, 0x3141, 0x3142, 0x3143, 0x3145,
    0x3146, 0x3147, 0x3148, 0x3149, 0x314a, 0x314b, 0x314c, 0x314d, 0x314e
]))
MEDIALS = list(map(lambda x: chr(x), [
    0x314f, 0x3150, 0x3151, 0x3152, 0x3153, 0x3154, 0x3155, 0x3156, 0x3157, 0x3158,
    0x3159, 0x315a, 0x315b, 0x315c, 0x315d, 0x315e, 0x315f, 0x3160, 0x3161, 0x3162,
    0x3163
]))
FINALS = list(map(lambda x: chr(x), [
    0x3131, 0x3132, 0x3133, 0x3134, 0x3135, 0x3136, 0x3137, 0x3139, 0x313a, 0x313b,
    0x313c, 0x313d, 0x313e, 0x313f, 0x3140, 0x3141, 0x3142, 0x3144, 0x3145, 0x3146,
    0x3147, 0x3148, 0x314a, 0x314b, 0x314c, 0x314d, 0x314e
]))


def check_syllable(x):
    return 0xAC00 <= ord(x) <= 0xD7A3


def split_syllable_char(x):
    if len(x) != 1:
        raise ValueError("Input string must have exactly one character.")

    if not check_syllable(x):
        raise ValueError("Input string does not contain a valid Korean character.")

    diff = ord(x) - 0xAC00
    _m = diff % 28
    _d = (diff - _m) // 28

    initial_index = _d // 21
    medial_index = _d % 21
    final_index = _m

    if not final_index:
        result = (INITIALS[initial_index], MEDIALS[medial_index])
    else:
        result = (INITIALS[initial_index], MEDIALS[medial_index], FINALS[final_index - 1])

    return result


def split_syllables(string):
    new_string = ""
    for c in string:
        if not check_syllable(c):
            new_c = c
        else:
            new_c = "".join(split_syllable_char(c))
        new_string += new_c

    return new_string


# ----------
# 한글-영어 변환
# ----------
KOR_WORDS = list('ㄱㄲㄳㄴㄵㄶㄷㄹㄺㄻㄼㄽㄾㄿㅀㅁㅂㅄㅅㅆㅇㅈㅊㅋㅌㅍㅎㅏㅐㅑㅒㅓㅔㅕㅖㅗㅘㅙㅚㅛㅜㅝㅞㅟㅠㅡㅢㅣ')
ENG_WORDS = [
    'r', 'R', 'rt', 's', 'sw', 'sg', 'e', 'f', 'fr', 'fa', 'fq', 'ft',
    'fx', 'fv', 'fg', 'a', 'q', 'qt', 't', 'T', 'd', 'w', 'c', 'z',
    'x', 'v', 'g', 'k', 'o', 'i', 'O', 'j', 'p', 'u', 'P', 'h', 'hk',
    'ho', 'hl', 'y', 'n', 'nj', 'np', 'nl', 'b', 'm', 'ml', 'l'
]
KOR_ENG_TABLE = dict(zip(KOR_WORDS, ENG_WORDS))



from os import environ

from korea_public_data.service.app import PublicDataController


def test_request_apt_trade_api():
    """일별 박스 오피스 순위 API"""
    # 앱 실행
    key = environ.get("KOBIS_OR_KR_KEY")
    app_name = 'searchDailyBoxOfficeList'

    if key:
        # 기본 사업자 조회
        controller = PublicDataController(app_name).set_keys(
            key=key
        )
        res = controller.data
        assert res.get("boxOfficeResult").get("boxofficeType") == "일별 박스오피스"



from korea_public_data.core.choices import ResponseType
from korea_public_data.core.vars import default as var
from korea_public_data.core.consts import data as const
from korea_public_data.data.base import PublicDataBase


class SearchDailyBoxOfficeList(PublicDataBase):
    """일별 박스 오피스 순위"""
    def __init__(self, key: str):
        # 상위 클래스 변수 적용
        super().__init__()
        # 필수 설정
        self.key = key
        # 변수(기본 값 적용)
        self.yesterday = var.DEFAULT_YESTERDAY_SEOUL_TIMEZONE
        self.target_dt = self.yesterday.strftime("%Y%m%d")
        self.item_per_page = var.DEFAULT_ITEM_PER_PAGE
        self.multi_movie_yn = var.DEFAULT_MULTI_MOVIE_YN
        self.rep_nation_cd = var.DEFAULT_REP_NATION_CD
        self.wide_area_cd = var.DEFAULT_WIDE_AREA_CD
        # 상수
        self.response_type = ResponseType.JSON

    @property
    def url(self):
        multi_movie_yn = f'multiMovieYn={self.multi_movie_yn}&' if self.multi_movie_yn else ''
        rep_nation_cd = f'repNationCd={self.rep_nation_cd}&' if self.rep_nation_cd else ''
        wide_area_cd = f'wideAreaCd={self.wide_area_cd}&' if self.wide_area_cd else ''
        return (
            f'{const.SEARCH_DAILY_BOX_OFFICE_LIST_URL}'
            f'key={self.key}&'
            f'targetDt={self.target_dt}&'
            f'{multi_movie_yn}'
            f'{rep_nation_cd}'
            f'{wide_area_cd}'
            f'itemPerPage={self.item_per_page}'
        )

    def _data_valid(self):
        """데이터 이상여부 확인"""
        assert self.key, "서비스 키가 등록되지 않았습니다."



from korea_public_data.core.choices import ResponseType
from korea_public_data.core.consts import data as const
from korea_public_data.data.base import PublicDataBase


class SOCityStats(PublicDataBase):
    """여성가족부_성범죄자 지역별 통계"""
    def __init__(self):
        # 상위 클래스 변수 적용
        super().__init__()
        # 상수
        self.response_type = ResponseType.XML

    @property
    def url(self):
        return (
            f'{const.SO_CITY_STATS_URL}'
        )

    def _data_valid(self):
        """데이터 이상여부 확인"""
        pass



from korea_public_data.data.base import PublicDataBase
from korea_public_data.data import *


class PublicDataController:
    def __init__(
            self,
            app: str,  # 앱 이름
    ):
        assert app and type(app) == str and len(app) > 0, "앱 이름이 없습니다."
        # 앱 이름 설정
        app_name = str(app[0]).upper() + app[1:]
        # 데이터 클래스 조회
        data_classes = {cls.__name__: cls for cls in PublicDataBase.__subclasses__()}
        self._data_class = data_classes.get(app_name)
        assert self._data_class, "선택하신 앱이 존재하지 않습니다."

    def set_keys(self, *args, **kwargs):
        """각 클래스의 키 값을 입력"""
        return self._data_class(*args, **kwargs)
